%macro rmpo_ddl_master(LIBREF=, DTTMFMT=NLDATM21., DTFMT=DATE9.);
/*
1. define boolean type for flag e.g. task_active ( using  0/1 );
2. define several id length based on different type of ids
3. do we need to add check constraint or this has been built into the etl?
4. add extra column in mkt_seg is_used to indicate if the mkt_seg is used. (is this change ok?)
5. define a
*/
%local boolean vshort short long id_len s_id_len v_id_len ;
/* a boolean to be a 0/1 integer */
%let boolean=3;
/* used for variable that has max value below 8192 */
%let vshort=3;
/* used for variable that has max value below 2,097,152 */
%let short=4;
%let id_len=8;
%let s_id_len=4;
%let v_id_len=3;
%let long=8;

/***********************************************************************************************/

/* RESOURCE_CAPACITY */
data &libref..RESOURCE_CAPACITY;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib RESOURCE_ID length=&id_len ;
attrib SERVICE_PERIOD length=&id_len ;
attrib EFFECTIVE_CAPACITY length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..RESOURCE_CAPACITY
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, RESOURCE_ID, SERVICE_PERIOD)
        ADD CONSTRAINT EFFECTIVE_CAPACITY CHECK(EFFECTIVE_CAPACITY >= 0);
quit;

/* UPGRADE_PATH */
data &libref..UPGRADE_PATH_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib FROM_RESOURCE_ID length=&id_len ;
attrib TO_RESOURCE_ID length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..UPGRADE_PATH_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, FROM_RESOURCE_ID,TO_RESOURCE_ID);
quit;


/* CUSTOMER_SEGMENT_RANK */
data &libref..CUSTOMER_SEGMENT_RANK_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib RESOURCE_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib RANK length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..CUSTOMER_SEGMENT_RANK_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, RESOURCE_ID, CUSTOMER_SEGMENT_ID)
        ADD NOT NULL (RANK);
quit;

/* SERVICE_LENGTH_CONFIG */
data &libref..SERVICE_LENGTH_CONFIG;
attrib SERVICE_LENGTH_CONFIG_ID length=&id_len ;
attrib SERVICE_LENGTH_ID length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..SERVICE_LENGTH_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (SERVICE_LENGTH_CONFIG_ID, SERVICE_LENGTH_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >=1);
quit;

/* COST_OF_WALK */
/* this table may not be populated for every combinations
 * assume overbooking_upper_bound = 0 for undefined combinations */
data &libref..COST_OF_WALK_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib RESOURCE_ID length=&id_len ;
attrib SERVICE_PERIOD length=&id_len ;
attrib COST_OF_WALK  length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..COST_OF_WALK_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, RESOURCE_ID, SERVICE_PERIOD)
        ADD CONSTRAINT COST_OF_WALK CHECK(COST_OF_WALK >= 0);
quit;

/* OVERBOOKING_UPGRADE_CONFIG */
/* This table may not be populated for all combinations.
 * Assume overbooking_upper_bound = -1, overbooking_override = -1, upgrade_upper_bound = -1 for undefined combinations */
data &libref..OVERBOOKING_UPGRADE_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib RESOURCE_ID length=&id_len ;
attrib SERVICE_PERIOD length=&id_len ;
attrib OVERBOOKING_UPPER_BOUND length=&id_len ;
attrib OVERBOOKING_OVERRIDE length=&id_len ;
attrib UPGRADE_UPPER_BOUND length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..OVERBOOKING_UPGRADE_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, RESOURCE_ID, SERVICE_PERIOD)
        ADD CONSTRAINT OVERBOOKING_UPPER_BOUND CHECK(OVERBOOKING_UPPER_BOUND >= 0 OR OVERBOOKING_UPPER_BOUND = -1)
        ADD CONSTRAINT OVERBOOKING_OVERRIDE CHECK(OVERBOOKING_OVERRIDE >= 0 OR OVERBOOKING_OVERRIDE = -1)
        ADD CONSTRAINT UPGRADE_UPPER_BOUND CHECK(UPGRADE_UPPER_BOUND >= 0 OR UPGRADE_UPPER_BOUND = -1);
quit;


/* PRODUCT_DEFINITION */
data &libref..PRODUCT_DEFINITION;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib RESOURCE_ID length=&id_len ;
attrib QUANTITY  length=&id_len ;
attrib REVENUE_PCT length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..PRODUCT_DEFINITION
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, PRODUCT_ID, RESOURCE_ID)
        ADD CONSTRAINT QUANTITY CHECK(QUANTITY >= 1)
        ADD CONSTRAINT REV_PCT CHECK(REVENUE_PCT >= 0 AND REVENUE_PCT <= 100);
quit;


/* BOOKING_WINDOW_CONFIG */
data &libref..BOOKING_WINDOW_CONFIG;
attrib BOOKING_WINDOW_CONFIG_ID length=&id_len ;
attrib BOOKING_WINDOW_ID length=&id_len ;
attrib BOOKING_WINDOW_START length=&id_len ;
attrib BOOKING_WINDOW_END length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..BOOKING_WINDOW_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (BOOKING_WINDOW_CONFIG_ID, BOOKING_WINDOW_ID)
        ADD NOT NULL (BOOKING_WINDOW_START)
        ADD NOT NULL (BOOKING_WINDOW_END)
        add constraint booking_window_id  check (booking_window_id >0)
        ADD CONSTRAINT START_LT_END CHECK(BOOKING_WINDOW_START <= BOOKING_WINDOW_END);
quit;


/* PRICE_GRID_FORECAST*/
data &libref..PRICE_GRID_FORECAST;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib SERVICE_START length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib PRICE_ID length=&id_len ;
attrib PRICE length=&id_len ;
stop;run;
proc sql;
    ALTER TABLE &libref..PRICE_GRID_FORECAST
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, CUSTOMER_SEGMENT_ID, PRODUCT_ID, SERVICE_START, SERVICE_LENGTH, PRICE_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1)
        ADD CONSTRAINT PRICE CHECK(PRICE >= 0);
quit;


/* PRICE_GRID */
data &libref..PRICE_GRID;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib SERVICE_START length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib PRICE_ID length=&id_len ;
attrib PRICE length=&id_len ;
stop;run;
proc sql;
    ALTER TABLE &libref..PRICE_GRID
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, CUSTOMER_SEGMENT_ID, PRODUCT_ID, SERVICE_START, SERVICE_LENGTH, PRICE_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1)
        ADD CONSTRAINT PRICE CHECK(PRICE >= 0);
quit;



/* PRICE_OVERRIDE */
data &libref..PRICE_OVERRIDE;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib SERVICE_START length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib PRICE_ID length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..PRICE_OVERRIDE
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, CUSTOMER_SEGMENT_ID, PRODUCT_ID, SERVICE_START, SERVICE_LENGTH, PRICE_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1);
quit;


/* DEMAND_FORECAST */
data &libref..DEMAND_FORECAST;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib SERVICE_START length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib BOOKING_WINDOW_ID length=&id_len ;
attrib PRICE_ID length=&id_len ;
attrib DEMAND length=&id_len ;
attrib DEMAND_STD length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..DEMAND_FORECAST
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, CUSTOMER_SEGMENT_ID, PRODUCT_ID, SERVICE_START, SERVICE_LENGTH, BOOKING_WINDOW_ID, PRICE_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1)
        ADD CONSTRAINT DEMAND CHECK(DEMAND >= 0)
        ADD CONSTRAINT DEMAND_STD CHECK(DEMAND_STD >= 0);
quit;

/* DECISION_CONTROL */
data &libref..DECISION_CONTROL_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib PRODUCT_ID  length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib CONTROL_GROUP_ID length=&id_len ;
attrib CONTROL_TYPE  length=$&s_id_len ;
stop; run;

proc sql;
    ALTER TABLE &libref.. DECISION_CONTROL_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, PRODUCT_ID, CUSTOMER_SEGMENT_ID)
        ADD NOT NULL (CONTROL_GROUP_ID)
        ADD CONSTRAINT CTRL_TYPE CHECK(CONTROL_TYPE IN ('AY' 'AN' 'LS' 'LY' 'NS'));
quit;

/* CANCEL_NOSHOW_FORECAST */
data &libref..CANCEL_NOSHOW_FORECAST;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib PRODUCT_ID  length=&id_len ;
attrib CUSTOMER_SEGMENT_ID  length=&id_len ;
attrib SERVICE_START  length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib BOOKING_WINDOW_ID length=&id_len ;
attrib CANCEL_RATE length=&id_len ;
attrib NOSHOW_RATE length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..CANCEL_NOSHOW_FORECAST
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, PRODUCT_ID, CUSTOMER_SEGMENT_ID, SERVICE_START, SERVICE_LENGTH, BOOKING_WINDOW_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1)
        ADD CONSTRAINT CANCEL_RATE CHECK(CANCEL_RATE >=0 AND CANCEL_RATE <= 1)
        ADD CONSTRAINT NOSHOW_RATE CHECK(NOSHOW_RATE >=0 AND NOSHOW_RATE <= 1);
quit;


/* CANCEL_NOSHOW_FORECAST */
data &libref..CANCEL_NOSHOW_REFUND_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib PRODUCT_ID  length=&id_len ;
attrib CUSTOMER_SEGMENT_ID  length=&id_len ;
attrib SERVICE_START  length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib BOOKING_WINDOW_ID length=&id_len ;
attrib CANCEL_REFUND_PCT length=&id_len ;
attrib NOSHOW_REFUND_PCT length=&id_len ;
stop;run;
/* delete this based on the conversation with sumanth and Kevin, will add this in the dq
proc sql;
    ALTER TABLE &libref..CANCEL_NOSHOW_REFUND_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, PRODUCT_ID, CUSTOMER_SEGMENT_ID, SERVICE_START, SERVICE_LENGTH, BOOKING_WINDOW_ID)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1)
        ADD CONSTRAINT CANCEL_REFUND_PCT CHECK(CANCEL_REFUND_PCT >=0 AND CANCEL_REFUND_PCT <= 100)
        ADD CONSTRAINT NOSHOW_REFUND_PCT CHECK(NOSHOW_REFUND_PCT >=0 AND NOSHOW_REFUND_PCT <= 100);
quit;
*/
/* SOLD_INFORMATION*/
data &libref..SOLD_INFORMATION;
attrib FORECAST_INSTANCE_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib SERVICE_START length=&id_len ;
attrib SERVICE_LENGTH length=&id_len ;
attrib SOLD length=&id_len ;
attrib REVENUE length=&id_len ;
attrib CANCEL_RATE length=&id_len ;
attrib NOSHOW_RATE length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..SOLD_INFORMATION
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (FORECAST_INSTANCE_ID, PRODUCT_ID, CUSTOMER_SEGMENT_ID, SERVICE_START, SERVICE_LENGTH)
        ADD CONSTRAINT SERVICE_LENGTH CHECK(SERVICE_LENGTH >= 1)
        ADD CONSTRAINT SOLD CHECK(SOLD >= 0)
        ADD CONSTRAINT REVENUE CHECK(REVENUE >= 0)
        ADD CONSTRAINT CANCEL_RATE CHECK(CANCEL_RATE >= 0 AND CANCEL_RATE <= 1)
        ADD CONSTRAINT NOSHOW_RATE CHECK(NOSHOW_RATE >= 0 AND NOSHOW_RATE <= 1);
quit;


/* CANCEL_REBOOK_FCST */
data &libref..CANCEL_REBOOK_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib PRODUCT_ID length=&id_len ;
attrib CUSTOMER_SEGMENT_ID length=&id_len ;
attrib CANCEL_REBOOK_RATE length=&id_len ;
attrib PENALTY length=&id_len ;
attrib PENALTY_TYPE length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..CANCEL_REBOOK_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, PRODUCT_ID, CUSTOMER_SEGMENT_ID)
        ADD CONSTRAINT CANCEL_REBOOK_RATE CHECK(CANCEL_REBOOK_RATE >= 0 AND CANCEL_REBOOK_RATE <= 1)
        ADD NOT NULL (PENALTY)
        ADD CONSTRAINT PENALTY_TYPE CHECK(PENALTY_TYPE IN (1 2)); /* 1=percentage; 2=dollar value */
quit;


/* OCCUPANCY_CONSTRAINT */
data &libref..OCCUPANCY_TARGET_CONFIG;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib SERVICE_PERIOD length=&id_len ;
attrib RESOURCE_ID length=&id_len ;
attrib OCCUPANCY_PCT length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..OCCUPANCY_TARGET_CONFIG
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID, SERVICE_PERIOD, RESOURCE_ID)
        ADD CONSTRAINT OCCUPANCY_PCT CHECK(OCCUPANCY_PCT >= 0 AND OCCUPANCY_PCT <= 100);
quit;


/* INSTANCE_PARAMETER */
data &libref..INSTANCE_PARAMETER;
attrib INSTANCE_ID length=&id_len ;
attrib INSTANCE_TYPE length=&vshort ;
attrib PARAMETER_NAME length=$32;
attrib PARAMETER_VALUE length=$1024;
stop;run;

proc sql noprint;
	alter table &libref..INSTANCE_PARAMETER
			ADD CONSTRAINT PRIM_KEY PRIMARY KEY (INSTANCE_ID, INSTANCE_TYPE, PARAMETER_NAME)
			ADD CONSTRAINT  INSTANCE_TYPE check(INSTANCE_TYPE in (1,2));
			/* 1 means forecast_instance_id, 2 means optimization_instance_id */
quit;

/* INSTANCE_MAP */
data &libref..INSTANCE_MAP;
attrib OPTIMIZATION_INSTANCE_ID length=&id_len ;
attrib FORECAST_INSTANCE_ID length=&id_len ;
attrib SERVICE_START length=&id_len ;
stop;run;

proc sql;
    ALTER TABLE &libref..INSTANCE_MAP
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (OPTIMIZATION_INSTANCE_ID);
quit;

data &libref..customer_segment_product_attr;
attrib FORECAST_INSTANCE_ID length=&id_len ;
attrib product_id length=&id_len ;
attrib customer_segment_id length=&id_len;
attrib priceable length=&vshort;
stop;
run;
proc sql;
    ALTER TABLE &libref..customer_segment_product_attr
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (FORECAST_INSTANCE_ID,product_id,customer_segment_id)
        ADD CONSTRAINT PRICEABLE CHECK(priceable in (0, 1));
quit;


data &libref..bid_price;
attrib optimization_instance_id length=&id_len ;
attrib resource_id length=&id_len ;
attrib service_period length=&id_len;
attrib authorized_capacity length=8;
attrib bid_price length=8;
attrib bid_price_delta length=8;
attrib capacity_shift length=8;
attrib ceiling length=8;
stop;
run;

data &libref..bid_price_vector;
attrib optimization_instance_id length=&id_len ;
attrib resource_id length=&id_len ;
attrib service_period length=&id_len;
attrib authorized_capacity length=8;
attrib bid_price length=8;
stop;
run;

data &libref..booking_pace_forecast;
attrib optimization_instance_id length=&id_len;
attrib customer_segment_id length=&id_len;
attrib product_id length=&id_len;
attrib booking_window_id length=&id_len;
attrib price_id length=&id_len;
attrib cumulative_demand length=8;
stop;
run;

data &libref..competitor;
attrib competitor_id length=&id_len;
attrib forecast_instance_id length=&id_len;
attrib self length=&vshort;
stop;
run;
proc sql;
    ALTER TABLE &libref..competitor
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (competitor_id)
        ADD CONSTRAINT SELF CHECK(self in (0, 1));
quit;

data &libref..competitor_rate_pace;
attrib competitor_id length=&id_len;
attrib forecast_instance_id length=&id_len;
attrib customer_segment_id length=&id_len;
attrib product_id length=&id_len;
attrib service_start length=8;
attrib service_length length=4;
attrib time_to_service length=8;
attrib channel_id length=8;
attrib competitor_rate length=8;
attrib available length=$3;
stop;
run;

data &libref..customer_segment_dim;
 attrib customer_segment_id length=&id_len;
 attrib customer_segment_cd length=&id_len;
 attrib customer_segment_desc length=$128;
stop;
run;

data &libref..dow_config;
  attrib dow_config_id length=&id_len;
  attrib dow_id length=&id_len;
  attrib dow length=&vshort;
stop;
run;
proc sql;
    ALTER TABLE &libref..dow_config
        ADD CONSTRAINT PRIM_KEY PRIMARY KEY (dow_config_id ,dow_id )
        ADD CONSTRAINT dow_id CHECK(dow_id >0 )
        ADD CONSTRAINT dow CHECK(dow between 1 and 7 )
        ;
quit;

data &libref..forecast_instance_dim;
 attrib forecast_instance_id length=&id_len;
 attrib forecast_instance_cd length=&id_len;
 attrib forecast_instance_desc length=$512;
stop;
run;


data &libref..forecast_method_override;
  attrib task_name length=$128;
  attrib macro_name length=$32;
  attrib status length=&vshort;
stop;
run;
proc sql;
    ALTER TABLE &libref..forecast_method_override
        ADD CONSTRAINT status CHECK(status in (0,1))
        ;
quit;

data &libref..model_parameter;
   attrib forecast_instance_id length=&id_len;
   attrib customer_segment_id length=&id_len;
   attrib product_id length=&id_len;
   attrib parameter_name length=$32;
   attrib parameter_value length=$1024;
stop;
run;

data &libref..occupancy_revenue_forecast;
  attrib optimization_instance_id length=&id_len;
  attrib resource_id length=&id_len;
  attrib customer_segment_id length=&id_len;
  attrib service_period length=8;
  attrib occupancy length=8;
  attrib revenue length=8;
stop;
run;

data &libref..overbooking;
  attrib optimization_instance_id length=&id_len;
  attrib resource_id length=&id_len;
  attrib service_period length=8;
  attrib capacity_shift_in length=8;
  attrib overbooking_limit length=8;
stop;
run;

data &libref..price_decision;
   attrib optimization_instance_id length=&id_len;
   attrib customer_segment_id length=&id_len;
   attrib product_id length=&id_len;
   attrib service_start length=8;
   attrib service_length length=4;
   attrib booking_window_id length=&id_len;
   attrib price_id length=&id_len;
   attrib price length=&id_len;
   attrib demand length=8;
stop;
run;

data &libref..product_dim;
  attrib product_id length=&id_len;
  attrib product_cd length=&id_len;
  attrib product_desc length=$128;
stop;
run;

data &libref..product_sold_pace;
   attrib forecast_instance_id length=&id_len;
   attrib customer_segment_id length=&id_len;
   attrib product_id length=&id_len;
   attrib service_start length=&id_len;
   attrib service_length length=4;
   attrib time_to_service length=8;
   attrib cumulative_sold length=8;
   attrib incremental_sold length=8;
   attrib cumulative_revenue length=8;
   attrib incremental_revenue length=8;
   attrib cumulative_cancellations length=8;
   attrib incremental_cancellations length=8;
   attrib noshows length=8;
stop;
run;

data &libref..protection_level;
  attrib optimization_instance_id length=&id_len;
  attrib customer_segment_id length=&id_len;
  attrib resource_id length=&id_len;
  attrib product_id length=&id_len;
  attrib service_period length=8;
  attrib service_start length=8;
  attrib price_id length=&id_len;
  attrib protection_level length=&id_len;
stop;
run;

data &libref..remaining_demand_override;
   attrib forecast_instance_id length=&id_len;
   attrib customer_segment_id length=&id_len;
   attrib product_id length=&id_len;
   attrib service_start length=8;
   attrib service_length length=&id_len;
   attrib scale length=8;
stop;
run;
proc sql;
    ALTER TABLE &libref..remaining_demand_override
        ADD CONSTRAINT scale CHECK(scale >=0) ;
quit;
data &libref..resource_dim;
  attrib resource_id length=&id_len;
  attrib resource_cd length=&id_len;
  attrib resource_desc length=$128;
stop;
run;

data &libref..season_config;
   attrib season_config_id length=&id_len;
   attrib season_id length=&id_len;
   attrib season_start length=8;
   attrib season_end length=8;
stop;
run;
proc sql;
    ALTER TABLE &libref..season_config
        ADD CONSTRAINT season_id CHECK(season_id >0)
        ;
quit;
data &libref..special_event_config;
   attrib special_event_config_id length=&id_len;
   attrib event_id length=&id_len;
   attrib event_start length=8;
   attrib event_end length=8;
   attrib event_start_offset length=4;
   attrib event_end_offset length=4;
   attrib exclude length=&vshort;
stop;
run;
proc sql;
    ALTER TABLE &libref..special_event_config
        ADD CONSTRAINT event_id CHECK(event_id >0)
        ADD CONSTRAINT  exclude CHECK(exclude in (0,1))
        ADD CONSTRAINT event_start_offset CHECK(event_start_offset >0)
        ADD CONSTRAINT event_end_offset CHECK(event_end_offset >0)
        ;
quit;
data &libref..unconstrained_occupancy_forecast;
  attrib forecast_instance_id length=&id_len;
  attrib customer_segment_id length=&id_len;
  attrib product_id length=&id_len;
  attrib service_period length=8;
  attrib base_demand length=8;
stop;
run;


/***********************************************************************************************/


data &libref..version;
  version_id=1; output;
run;

%mend;
/*
%rmpo_ddl_master(libref=work, DTTMFMT=DATETIME21., DTFMT=DATE9.);
*/
